#!/bin/bash

[[ $1 = -v ]] && VERBOSE=1 || unset VERBOSE
[[ -z $VERBOSE ]] && exec 2>/dev/null
export VERBOSE

if [[ ! -w /etc/passwd ]] ; then
  echo "Only 'root' can execute this script."
  exit 1
fi

#
# Set epages environmental variables
#
SYS_SCRIPT=/etc/sysconfig/epages6
if [[ -f $SYS_SCRIPT ]] ; then
  cp -p $SYS_SCRIPT $SYS_SCRIPT.tmp \
  && sed 's/\(EPAGES_MYSQL_CONNECTION\)=\([a-z].*\)/\1="\2"/i' $SYS_SCRIPT.tmp > $SYS_SCRIPT \
  && rm -f $SYS_SCRIPT.tmp
  . $SYS_SCRIPT
fi
EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}

die() {
  touch $EPAGES/Setup/mysql.ini
  exit 0
}

unset VOID_VAR
void_var() {
  echo "Following variable value(s) may be incorrect:"
  for i in $@ ; do echo "  $i=\"$(eval echo '$'$i)\"" ; done
  echo "Check these variable(s) in $SYS_SCRIPT and rerun:"
  echo "  $0"
  VOID_VAR=1
}

RUN_SCRIPT=/etc/default/$EPAGES_PROJECT
ls $RUN_SCRIPT >/dev/null || die

cp -p $RUN_SCRIPT $RUN_SCRIPT.tmp \
&& sed 's/\(EPAGES_MYSQL_CONNECTION\)=\([a-z].*\)/\1="\2"/i' $RUN_SCRIPT.tmp > $RUN_SCRIPT \
&& rm -f $RUN_SCRIPT.tmp
. $RUN_SCRIPT

export EPAGES_INSTALL_MY_CNF
[[ -n $EPAGES_INSTALL_MY_CNF ]] && MY_CNF_ARG="-c $EPAGES_INSTALL_MY_CNF" || unset MY_CNF_ARG
#
# Read the connection settings from sysconfig file
#
get_mysql_option(){
  for group in $1 ; do
    result=$(my_print_defaults $MY_CNF_ARG $group | sed -n "s/^--$2=//p" | tail -n 1)
    [[ -n $result ]] && { echo -n $result; return; }
  done
  echo -n $3
}

MYSQL_DATADIR=$(get_mysql_option "server mysqld" datadir /var/lib/mysql)
MYSQL_SOCKET=$(get_mysql_option "client mysql server mysqld" socket $MYSQL_DATADIR/mysql.sock)
MYSQL_HOST=$(get_mysql_option "client mysql" host ${EPAGES_INSTALL_MYSQL_HOST:-$(hostname)})
MYSQL_PORT=$(get_mysql_option "client mysql server mysqld" port 3306)

#
# Set the mysql connection string based on the decision if we have a socket or TCP connection here.
#
if [[ -n $EPAGES_DISTRIBUTED_INSTALLATION && -z $EPAGES_INSTALL_MYSQL_CONNECTION ]] ; then
  EPAGES_INSTALL_MYSQL_CONNECTION=$MYSQL_HOST;$MYSQL_PORT
fi
connection=${EPAGES_INSTALL_MYSQL_CONNECTION:-$MYSQL_SOCKET}
[[ ${connection#/} != $connection ]] && unset USE_TCP || USE_TCP=1

if [[ -n $USE_TCP ]] ; then
  MYSQL_HOST=${connection%;*}
  [[ $MYSQL_HOST = $connection ]] || MYSQL_PORT=${connection##*;}
  MYSQL_CONNECTION_ARG="-h $MYSQL_HOST -P $MYSQL_PORT"
  export MYSQL_CONN="host=$MYSQL_HOST;port=$MYSQL_PORT"
  EPAGES_MYSQL_CONNECTION="$MYSQL_HOST;$MYSQL_PORT"
else # use socket
  MYSQL_SOCKET=$connection
  ls -d $MYSQL_SOCKET >/dev/null || void_var EPAGES_INSTALL_MYSQL_CONNECTION
  MYSQL_CONNECTION_ARG=-S$MYSQL_SOCKET
  export MYSQL_CONN="mysql_socket=$MYSQL_SOCKET"
  EPAGES_MYSQL_CONNECTION=$MYSQL_SOCKET
fi
MYSQL_SOCKET_ARG=-S$MYSQL_SOCKET

#
# Read the backup user credentials.
#
unset MYSQL_PASS_ARG
export MYSQL_USER=${EPAGES_INSTALL_MYSQL_USER:-root}
export MYSQL_PASS=$EPAGES_INSTALL_MYSQL_PASS
MYSQL_PASS_ARG=${MYSQL_PASS:+"-p'$MYSQL_PASS'"}

#
# Test the connection based on the connection string.
#
unset SUCCESS

# 1st try: connection, user and password
if [[ -z $SUCCESS && -z $VOID_VAR ]] ; then
  echo "mysql $MYSQL_CONNECTION_ARG -u$MYSQL_USER $MYSQL_PASS_ARG" >/tmp/proc.~$$~
  echo exit | eval mysql $MYSQL_CONNECTION_ARG -u$MYSQL_USER $MYSQL_PASS_ARG 2>>/tmp/proc.~$$~ && SUCCESS=1
  [[ -n $VERBOSE ]] && echo "-> $(cat /tmp/proc.~$$~)"
fi

# 2nd try: connection, user and default password
if [[ -z $SUCCESS && -z $VOID_VAR && -z $MYSQL_PASS ]] ; then
  [[ -n $VERBOSE ]] && echo "-> mysql $MYSQL_CONNECTION_ARG -u$MYSQL_USER -pepages"
  echo exit | eval mysql $MYSQL_CONNECTION_ARG -u$MYSQL_USER -pepages && SUCCESS=1
  [[ -n $SUCCESS ]] && { MYSQL_PASS=epages; MYSQL_PASS_ARG=-p$MYSQL_PASS; }
fi

# 3rd try: default socket, user and password
if [[ -z $SUCCESS && -z $VOID_VAR ]] ; then
  [[ -n $VERBOSE ]] && echo "-> mysql -u$MYSQL_USER $MYSQL_PASS_ARG"
  echo exit | eval mysql -u$MYSQL_USER $MYSQL_PASS_ARG && SUCCESS=1
  [[ -n $SUCCESS ]] && MYSQL_CONNECTION_ARG=
fi

# 4th try: default socket, user and default password
if [[ -z $SUCCESS && -z $VOID_VAR && -z $MYSQL_PASS ]] ; then
  [[ -n $VERBOSE ]] && echo "-> mysql -u$MYSQL_USER -pepages"
  if echo exit | eval mysql -u$MYSQL_USER -pepages ; then
    SUCCESS=1 ; MYSQL_CONNECTION_ARG=
    MYSQL_PASS=epages; MYSQL_PASS_ARG=-p$MYSQL_PASS
  fi
fi

# otherwise error
if [[ -z $SUCCESS && -z $VOID_VAR ]] ; then
  [[ -z $VERBOSE ]] && cat /tmp/proc.~$$~
  void_var EPAGES_INSTALL_MYSQL_CONNECTION EPAGES_INSTALL_MYSQL_USER EPAGES_INSTALL_MYSQL_PASS
fi
rm -f /tmp/proc.~$$~

#
# Database engine must be InnoDB
#
if [[ -z $VOID_VAR ]] ; then
  STATEMENT="SHOW ENGINES;"
  [[ -n $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG | grep -Pq '^InnoDB\\s+(YES|DEFAULT)\\s+'"
  echo "$STATEMENT" | eval mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG | grep -Pq '^InnoDB\s+(YES|DEFAULT)\s+' || {
    [[ -z $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG | grep -Pq '^InnoDB\\s+(YES|DEFAULT)\\s+'"
    echo "ERROR: Statement failed, database engine InnoDB not enabled."
    die
  }
fi

#
# Grant all privileges on *.* to $MYSQL_USER
#
if [[ -z $VOID_VAR ]] ; then
  STATEMENT="DELETE FROM mysql.user WHERE user = ''; FLUSH PRIVILEGES;"
  [[ -n $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG"
  echo "$STATEMENT" | eval mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG
  STATEMENT="SELECT user,host FROM mysql.user WHERE user = '';"
  [[ -n $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG"
  echo "$STATEMENT" | eval mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG > /tmp/pid.$$
  if [[ -s /tmp/pid.$$ ]] ; then
    [[ -z $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG"
    echo "ERROR: Statement failed, empty users still exist:"
    cat /tmp/pid.$$
    rm -f /tmp/pid.$$
    die
  fi
  rm -f /tmp/pid.$$
  [[ -z $MYSQL_PASS ]] && IDENTIFIED_BY= || IDENTIFIED_BY="IDENTIFIED BY '$MYSQL_PASS'"
  STATEMENT="GRANT ALL ON *.* TO '$MYSQL_USER' $IDENTIFIED_BY WITH GRANT OPTION;"
  [[ -n $VERBOSE ]] && echo "echo \"$STATEMENT\" | mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG"
  echo "$STATEMENT" | eval mysql $MYSQL_SOCKET_ARG -u$MYSQL_USER $MYSQL_PASS_ARG
fi

#
# Decide wether to use socket or port
#
if [[ -z $USE_TCP ]] ; then

  IPS=$(PATH=/sbin:/usr/sbin:$PATH ip addr show \
  | awk '/^[[:space:]]*inet/{sub(/\/.*/,"",$2); printf "%s ", $2}')

  if [[ -n $EPAGES_INSTALL_RR_DEDICATED ]] ; then
    expr " $IPS " : ".* $EPAGES_INSTALL_RR_DEDICATED " >/dev/null 2>&1 || USE_TCP=1
  fi

  if [[ -z $USE_TCP ]] ; then
    $EPAGES_PERL/bin/perl -e '
    use DE_EPAGES::WebInterface::API::Config qw (WebInterfaceConfig WebInterfaceConfigSectionIsHost);
    my $IniConfig = WebInterfaceConfig();
    foreach my $Section ( @{$IniConfig->sectionNames()} ) {
        next unless WebInterfaceConfigSectionIsHost($Section);
        next unless $IniConfig->get( $Section, "APPLICATIONSERVER" );
        next if $Section ~~ @ARGV;
        exit 123;
    }
    ' $IPS 2>/dev/null
    [[ $? -eq 123  ]] && USE_TCP=1
  fi

  if [[ -n $USE_TCP ]] ; then
    MYSQL_CONN="host=$MYSQL_HOST;port=$MYSQL_PORT"
    EPAGES_MYSQL_CONNECTION="$MYSQL_HOST;$MYSQL_PORT"
  fi
fi
[[ -n $VERBOSE ]] && echo "MYSQL_CONN=$MYSQL_CONN"
[[ -n $VERBOSE ]] && echo "EPAGES_INSTALL_MY_CNF=$EPAGES_INSTALL_MY_CNF"

[[ -n $USE_TCP ]] && EPAGES_INSTALL_MYSQL_TCPCONN=$EPAGES_MYSQL_CONNECTION \
|| EPAGES_INSTALL_MYSQL_TCPCONN="$MYSQL_HOST;$MYSQL_PORT"

#
# Set EPAGES_MYSQL_CONNECTION in /etc/default/epages6
#
PATH=/var/epages:$PATH:/var/tmp:"$EPAGES"/bin
if [ "$EPAGES_MYSQL_CONNECTION" = /var/lib/mysql/mysql.sock ] ; then
  [[ -n $VERBOSE ]] && echo "ini-config.sh -f $RUN_SCRIPT -k EPAGES_MYSQL_CONNECTION -v '' -i"
  ini-config.sh -f $RUN_SCRIPT -k EPAGES_MYSQL_CONNECTION -v '' -i
else
  [[ -n $VERBOSE ]] && echo "ini-config.sh -f $RUN_SCRIPT -k EPAGES_MYSQL_CONNECTION -v '\"$EPAGES_MYSQL_CONNECTION\"' -i"
  ini-config.sh -f $RUN_SCRIPT -k EPAGES_MYSQL_CONNECTION -v "\"$EPAGES_MYSQL_CONNECTION\"" -i
fi
if [[ "" = $(ini-config.sh -f $SYS_SCRIPT -k EPAGES_INSTALL_MYSQL_TCPCONN -g) ]] ; then
  [[ -n $VERBOSE ]] && echo "ini-config.sh -f $SYS_SCRIPT -k EPAGES_INSTALL_MYSQL_TCPCONN -v '\"$EPAGES_INSTALL_MYSQL_TCPCONN\"' -i"
  ini-config.sh -f $SYS_SCRIPT -k EPAGES_INSTALL_MYSQL_TCPCONN -v "\"$EPAGES_INSTALL_MYSQL_TCPCONN\"" -i
fi
if [[ "" = $(ini-config.sh -f $SYS_SCRIPT -k EPAGES_INSTALL_MYSQL_HOST -g) ]] ; then
  [[ -n $VERBOSE ]] && echo "ini-config.sh -f $SYS_SCRIPT -k EPAGES_INSTALL_MYSQL_HOST -v '\"$MYSQL_HOST\"' -i"
  ini-config.sh -f $SYS_SCRIPT -k EPAGES_INSTALL_MYSQL_HOST -v "\"$MYSQL_HOST\"" -i
fi

[[ -z $VOID_VAR ]] || rm -f $EPAGES/Setup/mysql.ini

#
# Rewrite the database .conf files to update the datasource string and the backup user credentials.
#
cd "$EPAGES_CONFIG/Database.d" 2>/dev/null || exit 0

for i in Backup ${EPAGES_BUSINESS_UNITS:-Site Store} ; do
  [[ $i = Backup && ! -f $i.conf ]] && continue
  [[ -f ${i#Flex}.conf ]] || cp -p Store.conf.ept ${i#Flex}.conf

  BU=$(echo ${i#Flex} | tr A-Z a-z)
  [[ $BU = site || $BU = store ]] && USR=usr_${BU}db || USR=$BU
  [[ $i = Backup ]] && USR=$MYSQL_USER
  ini-config.sh -f ${i#Flex}.conf -s Connection -k login -v "$USR" -i
  ini-config.sh -f ${i#Flex}.conf -s Connection -k datasource -v "dbi:mysql:$MYSQL_CONN" -i

  if [[ $i = Backup ]] ; then
    ENC=$($PERL -e "use DE_EPAGES::Database::API::Connection qw(EncryptDBPassword); print EncryptDBPassword('$MYSQL_PASS')")
    if [[ -z $ENC ]] ; then
      grep -v encpasswd $i.conf > $i.conf.$$ && cat $i.conf.$$ > $i.conf
      rm -f $i.conf.$$
      ini-config.sh -f $i.conf -s Connection -k passwd -v "$MYSQL_PASS" -i
    else
      ini-config.sh -f $i.conf -s Connection -k encpasswd -v "$ENC" -i
      ini-config.sh -f $i.conf -s Connection -k passwd -v "" -i
    fi
  else
    ini-config.sh -f ${i#Flex}.conf -s Connection -k Database -v "${BU}db" -i
    ini-config.sh -f ${i#Flex}.conf -s Connection -k StoreType -v "$i" -i
  fi
  if [[ -n $EPAGES_INSTALL_MY_CNF ]] ; then
    ini-config.sh -f ${i#Flex}.conf -s Connection -k MySQLConfigFile -v "$EPAGES_INSTALL_MY_CNF" -i
  fi
  chown ep_appl:epages ${i#Flex}.conf
done

exit 0
